package com.biganto.visual.roompark.conductor.dialogs

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import androidx.annotation.LayoutRes
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.BaseRoomParkActivity
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.di.dagger.ActivityModule
import com.biganto.visual.roompark.di.dagger.AppComponent
import com.biganto.visual.roompark.di.dagger.PerScreen
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.domain.model.bytesToSize
import com.biganto.visual.roompark.util.view_utils.snackbar.ISnackBarProvider
import com.bluelinelabs.conductor.Controller
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.clicks
import dagger.Binds
import dagger.BindsInstance
import dagger.Component
import dagger.Module
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

private const val TYPICAL_PLAN_SIZE = (1.2f*1024L*1024L).toLong()


class StartPlansDownloadingDialogController : Controller {

    constructor() : super()
    constructor(args: Bundle) : super(args)


    private val detachDisposable = CompositeDisposable()

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }


    @Inject
    lateinit var interactor: SettingsInteractor

    @Inject
    lateinit var file: FileModule

    lateinit var snackbar: ISnackBarProvider

    @Inject
    lateinit var rpActivity: RoomParkMainActivity

    override fun onContextAvailable(context: Context) {
        super.onContextAvailable(context)

        DaggerStartPlansDownloadingDialogScreenComponent.factory()
            .create(RoomParkApplication.component,activity as RoomParkMainActivity)
            .inject(this)
    }


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        snackbar = ActivityModule.provideSnackBar(rpActivity)



        file.freeSpace.bytesToSize().let {
            view.findViewById<MaterialTextView>(R.id.disk_size_textView)
                .text = resources?.getString(R.string.download_all_tours_disk_size_title,it)
        }

        view.findViewById<Button>(R.id.alert_dismiss_button)
            .setOnClickListener { handleBack() }

        view.findViewById<ImageView>(R.id.close_current_button)
            .setOnClickListener { handleBack() }

        detachDisposable.add(
            interactor.fetchPlanTypesSizes()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe ({
                    view.findViewById<MaterialTextView>(R.id.all_tours_size_textView)
                        .text = resources?.getString(R.string.download_all_plans_size_title
                        ,(it* TYPICAL_PLAN_SIZE).bytesToSize())
                },
            {err ->
                Timber.e(err)
                snackbar.showSnackBar(R.string.download_all_plans_errors_snackbar_message)

            })
        )

        detachDisposable.add(

            view.findViewById<Button>(R.id.alert_ok_button)
                .clicks()
                .debounce(200, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    router.replaceTopController(
                        RouterTransaction.with(DownloadPlansDialogController())
                        .pushChangeHandler(DialogChangeHandler())
                        .popChangeHandler(DialogChangeHandler())
                    )
                }
        )

        view.findViewById<View>(R.id.close_current_button).setOnClickListener { handleBack() }
//        view.setOnClickListener { handleBack() }

        return view
    }


    @LayoutRes
    fun getLayoutId() = R.layout.tours_download_dialog_screen

    override fun handleBack(): Boolean {
        return router.popCurrentController()
    }
}

@PerScreen
@Component(
    modules = [StartPlansDownloadingDialogScreenModule::class],
    dependencies = [AppComponent::class])
interface StartPlansDownloadingDialogScreenComponent {

    @Component.Factory
    interface Factory{
        fun create(
            appComponent: AppComponent
            ,@BindsInstance activity: RoomParkMainActivity
        ): StartPlansDownloadingDialogScreenComponent
    }

    fun inject(controller: StartPlansDownloadingDialogController)
}

@Module
abstract class StartPlansDownloadingDialogScreenModule{


    @PerScreen
    @Binds
    abstract fun provideActivity(activity: RoomParkMainActivity): BaseRoomParkActivity
}
